searchData={"items":[{"type":"module","title":"AgentForge.ExecutionStats","doc":"Provides functionality for collecting and analyzing execution statistics of flows.\nTracks metrics such as execution time, steps taken, and signal patterns.","ref":"AgentForge.ExecutionStats.html"},{"type":"function","title":"AgentForge.ExecutionStats.finalize/2","doc":"Finalizes the execution stats with the result and calculates elapsed time.","ref":"AgentForge.ExecutionStats.html#finalize/2"},{"type":"function","title":"Parameters - AgentForge.ExecutionStats.finalize/2","doc":"* `stats` - Current execution stats struct\n  * `result` - The final result of the flow execution","ref":"AgentForge.ExecutionStats.html#finalize/2-parameters"},{"type":"function","title":"Examples - AgentForge.ExecutionStats.finalize/2","doc":"iex> stats = AgentForge.ExecutionStats.new()\n    iex> final = AgentForge.ExecutionStats.finalize(stats, {:ok, \"success\"})\n    iex> final.complete and is_integer(final.elapsed_ms) and final.result == {:ok, \"success\"}\n    true","ref":"AgentForge.ExecutionStats.html#finalize/2-examples"},{"type":"function","title":"AgentForge.ExecutionStats.format_report/1","doc":"Formats the execution stats into a human-readable report.","ref":"AgentForge.ExecutionStats.html#format_report/1"},{"type":"function","title":"Examples - AgentForge.ExecutionStats.format_report/1","doc":"iex> stats = AgentForge.ExecutionStats.new()\n    iex> stats = AgentForge.ExecutionStats.finalize(stats, {:ok, \"success\"})\n    iex> report = AgentForge.ExecutionStats.format_report(stats)\n    iex> String.contains?(report, \"Total Steps: 0\") and String.contains?(report, \"Result: {:ok, \\\"success\\\"}\")\n    true","ref":"AgentForge.ExecutionStats.html#format_report/1-examples"},{"type":"function","title":"AgentForge.ExecutionStats.new/0","doc":"Creates a new execution stats struct with initial values.","ref":"AgentForge.ExecutionStats.html#new/0"},{"type":"function","title":"Examples - AgentForge.ExecutionStats.new/0","doc":"iex> stats = AgentForge.ExecutionStats.new()\n    iex> is_integer(stats.start_time) and stats.steps == 0\n    true","ref":"AgentForge.ExecutionStats.html#new/0-examples"},{"type":"function","title":"AgentForge.ExecutionStats.record_step/4","doc":"Records a step in the execution process, updating relevant statistics.","ref":"AgentForge.ExecutionStats.html#record_step/4"},{"type":"function","title":"Parameters - AgentForge.ExecutionStats.record_step/4","doc":"* `stats` - Current execution stats struct\n  * `handler_info` - Information about the handler being executed\n  * `signal` - The signal being processed\n  * `state` - Current state of the flow","ref":"AgentForge.ExecutionStats.html#record_step/4-parameters"},{"type":"function","title":"Examples - AgentForge.ExecutionStats.record_step/4","doc":"iex> stats = AgentForge.ExecutionStats.new()\n    iex> signal = %{type: :test, data: \"data\"}\n    iex> updated = AgentForge.ExecutionStats.record_step(stats, :test_handler, signal, %{})\n    iex> updated.steps == 1 and updated.signal_types == %{test: 1}\n    true","ref":"AgentForge.ExecutionStats.html#record_step/4-examples"},{"type":"type","title":"AgentForge.ExecutionStats.t/0","doc":"","ref":"AgentForge.ExecutionStats.html#t:t/0"},{"type":"behaviour","title":"AgentForge.Notification.Channel","doc":"Behaviour for notification channels in AgentForge.","ref":"AgentForge.Notification.Channel.html"},{"type":"callback","title":"AgentForge.Notification.Channel.name/0","doc":"","ref":"AgentForge.Notification.Channel.html#c:name/0"},{"type":"callback","title":"AgentForge.Notification.Channel.send/2","doc":"","ref":"AgentForge.Notification.Channel.html#c:send/2"},{"type":"module","title":"AgentForge.Notification.Channels.Console","doc":"Console notification channel for AgentForge.","ref":"AgentForge.Notification.Channels.Console.html"},{"type":"module","title":"AgentForge.Notification.Registry","doc":"Registry for notification channels.","ref":"AgentForge.Notification.Registry.html"},{"type":"function","title":"AgentForge.Notification.Registry.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AgentForge.Notification.Registry.html#child_spec/1"},{"type":"function","title":"AgentForge.Notification.Registry.get_channel/1","doc":"Gets a channel by name.","ref":"AgentForge.Notification.Registry.html#get_channel/1"},{"type":"function","title":"AgentForge.Notification.Registry.list_channels/0","doc":"Lists all registered channels.","ref":"AgentForge.Notification.Registry.html#list_channels/0"},{"type":"function","title":"AgentForge.Notification.Registry.register_channel/1","doc":"Registers a notification channel.","ref":"AgentForge.Notification.Registry.html#register_channel/1"},{"type":"function","title":"AgentForge.Notification.Registry.start_link/1","doc":"Starts the notification registry.","ref":"AgentForge.Notification.Registry.html#start_link/1"},{"type":"behaviour","title":"AgentForge.Plugin","doc":"Behaviour specification for AgentForge plugins.\n\nPlugins can extend the framework with additional tools, primitives, and notification channels.","ref":"AgentForge.Plugin.html"},{"type":"callback","title":"AgentForge.Plugin.init/1","doc":"Called when the plugin is loaded. Use this for initialization.","ref":"AgentForge.Plugin.html#c:init/1"},{"type":"callback","title":"AgentForge.Plugin.metadata/0","doc":"Returns metadata about the plugin.","ref":"AgentForge.Plugin.html#c:metadata/0"},{"type":"callback","title":"AgentForge.Plugin.register_channels/0","doc":"Called to register notification channels provided by this plugin.","ref":"AgentForge.Plugin.html#c:register_channels/0"},{"type":"callback","title":"AgentForge.Plugin.register_primitives/0","doc":"Called to register primitives provided by this plugin.","ref":"AgentForge.Plugin.html#c:register_primitives/0"},{"type":"callback","title":"AgentForge.Plugin.register_tools/1","doc":"Called to register tools provided by this plugin.","ref":"AgentForge.Plugin.html#c:register_tools/1"},{"type":"module","title":"AgentForge.PluginManager","doc":"Manages loading and activation of AgentForge plugins.","ref":"AgentForge.PluginManager.html"},{"type":"function","title":"AgentForge.PluginManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AgentForge.PluginManager.html#child_spec/1"},{"type":"function","title":"AgentForge.PluginManager.list_plugins/0","doc":"Returns a list of loaded plugins with their metadata.","ref":"AgentForge.PluginManager.html#list_plugins/0"},{"type":"function","title":"AgentForge.PluginManager.load_plugin/2","doc":"Loads a plugin module.","ref":"AgentForge.PluginManager.html#load_plugin/2"},{"type":"function","title":"AgentForge.PluginManager.plugin_loaded?/1","doc":"Returns whether a plugin is loaded.","ref":"AgentForge.PluginManager.html#plugin_loaded?/1"},{"type":"function","title":"AgentForge.PluginManager.start_link/1","doc":"Starts the plugin manager.","ref":"AgentForge.PluginManager.html#start_link/1"},{"type":"module","title":"AgentForge.Plugins.HTTP","doc":"HTTP integration plugin for AgentForge.\n\nProvides tools for making HTTP requests in workflows.","ref":"AgentForge.Plugins.HTTP.html"},{"type":"module","title":"AgentForge","doc":"AgentForge is a lightweight, signal-driven workflow framework designed for personal projects.","ref":"AgentForge.html"},{"type":"module","title":"Key Features - AgentForge","doc":"- Signal-based communication\n- Composable handler pipelines\n- Persistent state management\n- Debug tracing support","ref":"AgentForge.html#module-key-features"},{"type":"module","title":"Example - AgentForge","doc":"# Define a simple handler\n    message_handler = fn signal, state ->\n      new_state = Map.put(state, :last_message, signal.data)\n      {AgentForge.Signal.emit(:processed, \"Got: #{signal.data}\"), new_state}\n    end\n\n    # Create a flow with the handler\n    flow = AgentForge.new_flow([message_handler], debug: true)\n\n    # Execute the flow with a signal\n    {:ok, result, _state} = flow.(AgentForge.Signal.new(:message, \"Hello\"))\n    # result.data will be \"Got: Hello\"","ref":"AgentForge.html#module-example"},{"type":"function","title":"AgentForge.emit/3","doc":"","ref":"AgentForge.html#emit/3"},{"type":"function","title":"AgentForge.emit_many/1","doc":"","ref":"AgentForge.html#emit_many/1"},{"type":"function","title":"AgentForge.get_last_execution_stats/0","doc":"Gets statistics from the last flow execution.\nReturns nil if no flow has been executed yet or statistics collection was disabled.","ref":"AgentForge.html#get_last_execution_stats/0"},{"type":"function","title":"Examples - AgentForge.get_last_execution_stats/0","doc":"iex> handlers = [fn signal, state -> {{:emit, signal}, state} end]\n    iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {:ok, _, _} = AgentForge.process_with_limits(handlers, signal, %{})\n    iex> stats = AgentForge.get_last_execution_stats()\n    iex> stats.steps\n    1","ref":"AgentForge.html#get_last_execution_stats/0-examples"},{"type":"function","title":"AgentForge.halt/1","doc":"","ref":"AgentForge.html#halt/1"},{"type":"function","title":"AgentForge.new_flow/2","doc":"Creates a new flow with the given handlers and options.\nReturns a function that can be used to execute the flow.","ref":"AgentForge.html#new_flow/2"},{"type":"function","title":"Options - AgentForge.new_flow/2","doc":"All options from `AgentForge.Runtime.execute/3` are supported.","ref":"AgentForge.html#new_flow/2-options"},{"type":"function","title":"Examples - AgentForge.new_flow/2","doc":"iex> handler = fn signal, state ->\n    ...>   {AgentForge.Signal.emit(:echo, signal.data), state}\n    ...> end\n    iex> flow = AgentForge.new_flow([handler])\n    iex> {:ok, result, _} = flow.(AgentForge.Signal.new(:test, \"hello\"))\n    iex> result.data\n    \"hello\"","ref":"AgentForge.html#new_flow/2-examples"},{"type":"function","title":"AgentForge.new_signal/3","doc":"","ref":"AgentForge.html#new_signal/3"},{"type":"function","title":"AgentForge.new_stateful_flow/2","doc":"Creates a new stateful flow that maintains state between executions.\nSimilar to new_flow/2 but automatically stores and retrieves state.","ref":"AgentForge.html#new_stateful_flow/2"},{"type":"function","title":"Examples - AgentForge.new_stateful_flow/2","doc":"iex> {:ok, _pid} = AgentForge.Store.start_link(name: :test_store)\n    iex> counter = fn _signal, state ->\n    ...>   count = Map.get(state, :count, 0) + 1\n    ...>   {AgentForge.Signal.emit(:count, count), Map.put(state, :count, count)}\n    ...> end\n    iex> flow = AgentForge.new_stateful_flow([counter], store_name: :test_store)\n    iex> {:ok, result1, _} = flow.(AgentForge.Signal.new(:inc, nil))\n    iex> {:ok, result2, _} = flow.(AgentForge.Signal.new(:inc, nil))\n    iex> result2.data > result1.data\n    true","ref":"AgentForge.html#new_stateful_flow/2-examples"},{"type":"function","title":"AgentForge.process_with_limits/4","doc":"Processes a flow with execution limits.\nThis can prevent long-running operations.","ref":"AgentForge.html#process_with_limits/4"},{"type":"function","title":"Options - AgentForge.process_with_limits/4","doc":"* `:timeout_ms` - Maximum execution time in milliseconds (default: 30000)\n* `:collect_stats` - Whether to collect execution statistics (default: true)\n* `:return_stats` - Whether to return statistics in the result (default: false)","ref":"AgentForge.html#process_with_limits/4-options"},{"type":"function","title":"Examples - AgentForge.process_with_limits/4","doc":"iex> handlers = [\n    ...>   fn _signal, state -> {{:emit, AgentForge.Signal.new(:done, \"Success\")}, state} end\n    ...> ]\n    iex> {:ok, result, _} = AgentForge.process_with_limits(handlers, AgentForge.Signal.new(:test, \"data\"), %{})\n    iex> result.data\n    \"Success\"","ref":"AgentForge.html#process_with_limits/4-examples"},{"type":"module","title":"AgentForge.Flow","doc":"Provides functions for processing signals through a chain of handlers.\nEach handler is a function that takes a signal and state, and returns a tuple with result and new state.","ref":"AgentForge.Flow.html"},{"type":"function","title":"AgentForge.Flow.always_emit/2","doc":"Creates a handler that always emits a signal of the given type and data.","ref":"AgentForge.Flow.html#always_emit/2"},{"type":"function","title":"Examples - AgentForge.Flow.always_emit/2","doc":"iex> handler = AgentForge.Flow.always_emit(:done, \"success\")\n    iex> {result, state} = handler.(nil, %{})\n    iex> match?({:emit, %{type: :done, data: \"success\"}}, result)\n    true","ref":"AgentForge.Flow.html#always_emit/2-examples"},{"type":"function","title":"AgentForge.Flow.filter_type/2","doc":"Creates a handler that only processes signals of a specific type.\nOther signal types are skipped.","ref":"AgentForge.Flow.html#filter_type/2"},{"type":"function","title":"Examples - AgentForge.Flow.filter_type/2","doc":"iex> inner = fn signal, state -> {AgentForge.Signal.emit(:processed, signal.data), state} end\n    iex> handler = AgentForge.Flow.filter_type(:test, inner)\n    iex> test_signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {result, _} = handler.(test_signal, %{})\n    iex> match?({:emit, %{type: :processed}}, result)\n    true\n    iex> other_signal = AgentForge.Signal.new(:other, \"data\")\n    iex> handler.(other_signal, %{}) |> elem(0)\n    :skip","ref":"AgentForge.Flow.html#filter_type/2-examples"},{"type":"type","title":"AgentForge.Flow.flow/0","doc":"A flow is a handler function or a list of handler functions.\nEach handler takes a signal and state, and returns a tuple with result and new state.","ref":"AgentForge.Flow.html#t:flow/0"},{"type":"function","title":"AgentForge.Flow.get_last_execution_stats/0","doc":"","ref":"AgentForge.Flow.html#get_last_execution_stats/0"},{"type":"function","title":"AgentForge.Flow.process/3","doc":"","ref":"AgentForge.Flow.html#process/3"},{"type":"function","title":"AgentForge.Flow.process_handler/3","doc":"","ref":"AgentForge.Flow.html#process_handler/3"},{"type":"function","title":"AgentForge.Flow.process_with_limits/4","doc":"Processes a signal through a list of handlers with execution limits.\nSupports timeout to prevent long-running processes.","ref":"AgentForge.Flow.html#process_with_limits/4"},{"type":"function","title":"Options - AgentForge.Flow.process_with_limits/4","doc":"* `:timeout_ms` - Maximum time in milliseconds to process (default: 30000)\n* `:collect_stats` - Whether to collect execution statistics (default: true)\n* `:return_stats` - Whether to return statistics in the result (default: false)","ref":"AgentForge.Flow.html#process_with_limits/4-options"},{"type":"function","title":"Examples - AgentForge.Flow.process_with_limits/4","doc":"iex> handlers = [\n    ...>   fn sig, st -> {{:emit, AgentForge.Signal.new(:echo, sig.data)}, st} end\n    ...> ]\n    iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {:ok, result, _} = AgentForge.Flow.process_with_limits(handlers, signal, %{})\n    iex> result.type\n    :echo\n    \nWith statistics:\n\n    iex> handlers = [\n    ...>   fn sig, st -> {{:emit, AgentForge.Signal.new(:echo, sig.data)}, st} end\n    ...> ]\n    iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {:ok, _result, _, stats} = AgentForge.Flow.process_with_limits(handlers, signal, %{}, return_stats: true)\n    iex> stats.steps >= 1\n    true","ref":"AgentForge.Flow.html#process_with_limits/4-examples"},{"type":"function","title":"AgentForge.Flow.store_in_state/1","doc":"Creates a handler that stores the signal data in state under the given key.","ref":"AgentForge.Flow.html#store_in_state/1"},{"type":"function","title":"Examples - AgentForge.Flow.store_in_state/1","doc":"iex> handler = AgentForge.Flow.store_in_state(:last_message)\n    iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {result, state} = handler.(signal, %{})\n    iex> result\n    :skip\n    iex> state.last_message\n    \"data\"","ref":"AgentForge.Flow.html#store_in_state/1-examples"},{"type":"module","title":"AgentForge.Signal","doc":"Handles creation and manipulation of signals in the AgentForge system.\nSignals are the fundamental unit of communication.","ref":"AgentForge.Signal.html"},{"type":"function","title":"AgentForge.Signal.correlate/2","doc":"Creates a new signal correlated with a parent signal.\nCopies the parent's trace_id into the new signal's correlation_id.","ref":"AgentForge.Signal.html#correlate/2"},{"type":"function","title":"Examples - AgentForge.Signal.correlate/2","doc":"iex> parent = AgentForge.Signal.new(:parent, \"data\")\n    iex> child = AgentForge.Signal.correlate(AgentForge.Signal.new(:child, \"response\"), parent)\n    iex> child.meta.correlation_id == parent.meta.trace_id\n    true","ref":"AgentForge.Signal.html#correlate/2-examples"},{"type":"function","title":"AgentForge.Signal.emit/3","doc":"Creates an emit result with a new signal.","ref":"AgentForge.Signal.html#emit/3"},{"type":"function","title":"Examples - AgentForge.Signal.emit/3","doc":"iex> {:emit, signal} = AgentForge.Signal.emit(:test, \"data\")\n    iex> signal.type == :test and signal.data == \"data\"\n    true","ref":"AgentForge.Signal.html#emit/3-examples"},{"type":"function","title":"AgentForge.Signal.emit_many/1","doc":"Creates an emit_many result with multiple signals.","ref":"AgentForge.Signal.html#emit_many/1"},{"type":"function","title":"Examples - AgentForge.Signal.emit_many/1","doc":"iex> {:emit_many, signals} = AgentForge.Signal.emit_many([\n    ...>   {:test1, \"data1\"},\n    ...>   {:test2, \"data2\"}\n    ...> ])\n    iex> length(signals) == 2\n    true","ref":"AgentForge.Signal.html#emit_many/1-examples"},{"type":"function","title":"AgentForge.Signal.get_meta/2","doc":"Gets a metadata value from a signal.","ref":"AgentForge.Signal.html#get_meta/2"},{"type":"function","title":"Examples - AgentForge.Signal.get_meta/2","doc":"iex> signal = AgentForge.Signal.new(:test, \"data\", %{custom: %{key: \"value\"}})\n    iex> AgentForge.Signal.get_meta(signal, :custom)\n    %{key: \"value\"}","ref":"AgentForge.Signal.html#get_meta/2-examples"},{"type":"function","title":"AgentForge.Signal.halt/1","doc":"Creates a halt result with a value.","ref":"AgentForge.Signal.html#halt/1"},{"type":"function","title":"Examples - AgentForge.Signal.halt/1","doc":"iex> {:halt, value} = AgentForge.Signal.halt(\"done\")\n    iex> value == \"done\"\n    true","ref":"AgentForge.Signal.html#halt/1-examples"},{"type":"function","title":"AgentForge.Signal.new/3","doc":"Creates a new signal with the given type and data.","ref":"AgentForge.Signal.html#new/3"},{"type":"function","title":"Examples - AgentForge.Signal.new/3","doc":"iex> signal = AgentForge.Signal.new(:user_message, \"Hello\")\n    iex> signal.type == :user_message and signal.data == \"Hello\"\n    true","ref":"AgentForge.Signal.html#new/3-examples"},{"type":"type","title":"AgentForge.Signal.signal_result/0","doc":"","ref":"AgentForge.Signal.html#t:signal_result/0"},{"type":"type","title":"AgentForge.Signal.t/0","doc":"","ref":"AgentForge.Signal.html#t:t/0"},{"type":"function","title":"AgentForge.Signal.update_meta/3","doc":"Updates metadata in a signal.","ref":"AgentForge.Signal.html#update_meta/3"},{"type":"function","title":"Examples - AgentForge.Signal.update_meta/3","doc":"iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> updated = AgentForge.Signal.update_meta(signal, :custom, %{key: \"value\"})\n    iex> get_in(updated.meta, [:custom, :key])\n    \"value\"","ref":"AgentForge.Signal.html#update_meta/3-examples"},{"type":"module","title":"AgentForge.Store","doc":"A simple state container implementation using GenServer.\nProvides basic operations for state management.","ref":"AgentForge.Store.html"},{"type":"function","title":"AgentForge.Store.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AgentForge.Store.html#child_spec/1"},{"type":"function","title":"AgentForge.Store.delete/2","doc":"Deletes a value from the store.","ref":"AgentForge.Store.html#delete/2"},{"type":"function","title":"Examples - AgentForge.Store.delete/2","doc":"iex> {:ok, _} = AgentForge.Store.start_link(name: :test_store4)\n    iex> AgentForge.Store.delete(:test_store4, :counter)\n    :ok","ref":"AgentForge.Store.html#delete/2-examples"},{"type":"function","title":"AgentForge.Store.get/2","doc":"Gets a value from the store.","ref":"AgentForge.Store.html#get/2"},{"type":"function","title":"Examples - AgentForge.Store.get/2","doc":"iex> {:ok, _} = AgentForge.Store.start_link(name: :test_store)\n    iex> AgentForge.Store.put(:test_store, :counter, 42)\n    iex> AgentForge.Store.get(:test_store, :counter)\n    {:ok, 42}\n    iex> AgentForge.Store.get(:test_store, :nonexistent)\n    {:error, :not_found}","ref":"AgentForge.Store.html#get/2-examples"},{"type":"function","title":"AgentForge.Store.put/3","doc":"Puts a value in the store.","ref":"AgentForge.Store.html#put/3"},{"type":"function","title":"Examples - AgentForge.Store.put/3","doc":"iex> {:ok, _} = AgentForge.Store.start_link(name: :test_store2)\n    iex> AgentForge.Store.put(:test_store2, :counter, 42)\n    :ok","ref":"AgentForge.Store.html#put/3-examples"},{"type":"function","title":"AgentForge.Store.start_link/1","doc":"Starts the store process with an optional name.","ref":"AgentForge.Store.html#start_link/1"},{"type":"function","title":"AgentForge.Store.update/4","doc":"Updates a value in the store using a function.","ref":"AgentForge.Store.html#update/4"},{"type":"function","title":"Examples - AgentForge.Store.update/4","doc":"iex> {:ok, _} = AgentForge.Store.start_link(name: :test_store3)\n    iex> AgentForge.Store.update(:test_store3, :counter, 0, &(&1 + 1))\n    :ok","ref":"AgentForge.Store.html#update/4-examples"},{"type":"module","title":"AgentForge.Primitives","doc":"Provides basic primitives for building dynamic workflows.\nIncludes fundamental building blocks like branching, transformation, and iteration.","ref":"AgentForge.Primitives.html"},{"type":"function","title":"AgentForge.Primitives.branch/3","doc":"Creates a branch primitive that executes different handlers based on a condition.","ref":"AgentForge.Primitives.html#branch/3"},{"type":"function","title":"Options - AgentForge.Primitives.branch/3","doc":"* `:condition` - A function that takes a signal and state and returns a boolean\n* `:then_flow` - Flow to execute when condition is true\n* `:else_flow` - Flow to execute when condition is false","ref":"AgentForge.Primitives.html#branch/3-options"},{"type":"function","title":"Examples - AgentForge.Primitives.branch/3","doc":"iex> branch = AgentForge.Primitives.branch(\n    ...>   fn signal, _ -> signal.data > 10 end,\n    ...>   [fn signal, state -> {AgentForge.Signal.emit(:high, signal.data), state} end],\n    ...>   [fn signal, state -> {AgentForge.Signal.emit(:low, signal.data), state} end]\n    ...> )\n    iex> signal = AgentForge.Signal.new(:value, 15)\n    iex> {result, _} = branch.(signal, %{})\n    iex> match?({:emit, %{type: :high}}, result)\n    true","ref":"AgentForge.Primitives.html#branch/3-examples"},{"type":"function","title":"AgentForge.Primitives.loop/1","doc":"Creates a loop primitive that iterates over items in the signal data.\nAccumulates results and maintains state across iterations.","ref":"AgentForge.Primitives.html#loop/1"},{"type":"function","title":"Examples - AgentForge.Primitives.loop/1","doc":"iex> handler = fn item, state ->\n    ...>   {AgentForge.Signal.emit(:item, item), Map.update(state, :sum, item, &(&1 + item))}\n    ...> end\n    iex> loop = AgentForge.Primitives.loop(handler)\n    iex> signal = AgentForge.Signal.new(:list, [1, 2, 3])\n    iex> {result, state} = loop.(signal, %{})\n    iex> match?({:emit_many, _}, result) and state.sum == 6\n    true","ref":"AgentForge.Primitives.html#loop/1-examples"},{"type":"function","title":"AgentForge.Primitives.notify/2","doc":"Creates a notify primitive that sends notifications through configured channels.","ref":"AgentForge.Primitives.html#notify/2"},{"type":"function","title":"Options - AgentForge.Primitives.notify/2","doc":"* `:channels` - List of notification channels (e.g. [:console, :slack])\n* `:format` - Optional function to format the notification message\n* `:config` - Optional configuration map for channels","ref":"AgentForge.Primitives.html#notify/2-options"},{"type":"function","title":"Examples - AgentForge.Primitives.notify/2","doc":"iex> notify = AgentForge.Primitives.notify([:console])\n    iex> signal = AgentForge.Signal.new(:event, \"System alert\")\n    iex> {{:emit, result}, _} = notify.(signal, %{})\n    iex> result.type == :notification\n    true","ref":"AgentForge.Primitives.html#notify/2-examples"},{"type":"function","title":"AgentForge.Primitives.sequence/1","doc":"Creates a sequence primitive that executes multiple handlers in order,\npassing the result of each to the next.","ref":"AgentForge.Primitives.html#sequence/1"},{"type":"function","title":"Examples - AgentForge.Primitives.sequence/1","doc":"iex> handlers = [\n    ...>   fn signal, state -> {AgentForge.Signal.emit(:step1, signal.data <> \"_1\"), state} end,\n    ...>   fn signal, state -> {AgentForge.Signal.emit(:step2, signal.data <> \"_2\"), state} end\n    ...> ]\n    iex> sequence = AgentForge.Primitives.sequence(handlers)\n    iex> signal = AgentForge.Signal.new(:start, \"data\")\n    iex> {{:emit, result}, _} = sequence.(signal, %{})\n    iex> result.data\n    \"data_1_2\"","ref":"AgentForge.Primitives.html#sequence/1-examples"},{"type":"function","title":"AgentForge.Primitives.transform/1","doc":"Creates a transform primitive that modifies signal data.","ref":"AgentForge.Primitives.html#transform/1"},{"type":"function","title":"Examples - AgentForge.Primitives.transform/1","doc":"iex> transform = AgentForge.Primitives.transform(fn data -> String.upcase(data) end)\n    iex> signal = AgentForge.Signal.new(:text, \"hello\")\n    iex> {{:emit, result}, _} = transform.(signal, %{})\n    iex> result.data\n    \"HELLO\"","ref":"AgentForge.Primitives.html#transform/1-examples"},{"type":"function","title":"AgentForge.Primitives.wait/2","doc":"Creates a wait primitive that pauses execution until a condition is met.\nReturns {:wait, reason} while waiting, {:emit, signal} when condition is met.","ref":"AgentForge.Primitives.html#wait/2"},{"type":"function","title":"Options - AgentForge.Primitives.wait/2","doc":"* `:condition` - A function that takes a signal and state and returns a boolean\n* `:timeout` - Optional timeout in milliseconds, defaults to 5000\n* `:retry_interval` - Optional retry interval in milliseconds, defaults to 100","ref":"AgentForge.Primitives.html#wait/2-options"},{"type":"function","title":"Examples - AgentForge.Primitives.wait/2","doc":"iex> wait = AgentForge.Primitives.wait(\n    ...>   fn _, state -> Map.get(state, :ready, false) end,\n    ...>   timeout: 1000\n    ...> )\n    iex> signal = AgentForge.Signal.new(:check, \"waiting\")\n    iex> state = %{ready: false}\n    iex> {{:wait, reason}, _state} = wait.(signal, state)\n    iex> is_binary(reason)\n    true","ref":"AgentForge.Primitives.html#wait/2-examples"},{"type":"module","title":"AgentForge.DynamicFlow","doc":"Provides functions for dynamic flow creation and modification at runtime.\nAllows flows to adapt based on signal content and execution context.","ref":"AgentForge.DynamicFlow.html"},{"type":"function","title":"AgentForge.DynamicFlow.append_handlers/1","doc":"Creates a handler that appends additional handlers based on signal content.\nThe handlers_selector function should return a list of handlers to append.","ref":"AgentForge.DynamicFlow.html#append_handlers/1"},{"type":"function","title":"Examples - AgentForge.DynamicFlow.append_handlers/1","doc":"iex> selector = fn signal, _ ->\n    ...>   if String.contains?(signal.data, \"urgent\") do\n    ...>     [fn _s, st -> {AgentForge.Signal.emit(:priority, \"High priority\"), st} end]\n    ...>   else\n    ...>     [fn _s, st -> {AgentForge.Signal.emit(:priority, \"Normal priority\"), st} end]\n    ...>   end\n    ...> end\n    iex> handler = AgentForge.DynamicFlow.append_handlers(selector)\n    iex> signal = AgentForge.Signal.new(:message, \"urgent task\")\n    iex> {{:emit, result}, _state} = handler.(signal, %{})\n    iex> result.data\n    \"High priority\"","ref":"AgentForge.DynamicFlow.html#append_handlers/1-examples"},{"type":"function","title":"AgentForge.DynamicFlow.merge_flows/1","doc":"Creates a flow merger that combines multiple flows into one.\nEach flow is executed in sequence with the same input signal.\nResults are collected and emitted as a batch.","ref":"AgentForge.DynamicFlow.html#merge_flows/1"},{"type":"function","title":"Examples - AgentForge.DynamicFlow.merge_flows/1","doc":"iex> flows = [\n    ...>   [fn _s, st -> {AgentForge.Signal.emit(:result1, \"data_1\"), st} end],\n    ...>   [fn _s, st -> {AgentForge.Signal.emit(:result2, \"data_2\"), st} end]\n    ...> ]\n    iex> merger = AgentForge.DynamicFlow.merge_flows(flows)\n    iex> signal = AgentForge.Signal.new(:start, \"data\")\n    iex> {{:emit_many, results}, _state} = merger.(signal, %{})\n    iex> length(results) == 2\n    true","ref":"AgentForge.DynamicFlow.html#merge_flows/1-examples"},{"type":"function","title":"AgentForge.DynamicFlow.select_flow/1","doc":"Creates a handler that dynamically selects a flow based on signal content.\nThe flow_selector function should return a list of handlers.","ref":"AgentForge.DynamicFlow.html#select_flow/1"},{"type":"function","title":"Examples - AgentForge.DynamicFlow.select_flow/1","doc":"iex> selector = AgentForge.DynamicFlow.select_flow(fn signal, _ ->\n    ...>   case signal.type do\n    ...>     :text -> [fn _s, st -> {AgentForge.Signal.emit(:processed_text, signal.data), st} end]\n    ...>     :number -> [fn _s, st -> {AgentForge.Signal.emit(:processed_number, signal.data * 2), st} end]\n    ...>     _ -> [fn _s, st -> {AgentForge.Signal.emit(:unknown, signal.data), st} end]\n    ...>   end\n    ...> end)\n    iex> signal = AgentForge.Signal.new(:number, 5)\n    iex> {{:emit, result}, _state} = selector.(signal, %{})\n    iex> result.data\n    10","ref":"AgentForge.DynamicFlow.html#select_flow/1-examples"},{"type":"function","title":"AgentForge.DynamicFlow.subflow/1","doc":"Creates a handler that dynamically creates and executes a subflow.\nThe subflow_creator function should return a complete flow configuration.","ref":"AgentForge.DynamicFlow.html#subflow/1"},{"type":"function","title":"Examples - AgentForge.DynamicFlow.subflow/1","doc":"iex> creator = fn signal, _ ->\n    ...>   case signal.type do\n    ...>     :validate -> [\n    ...>       fn _s, st -> {AgentForge.Signal.emit(:validated, signal.data), st} end\n    ...>     ]\n    ...>     :process -> [\n    ...>       fn _s, st -> {AgentForge.Signal.emit(:processed, signal.data), st} end\n    ...>     ]\n    ...>   end\n    ...> end\n    iex> handler = AgentForge.DynamicFlow.subflow(creator)\n    iex> signal = AgentForge.Signal.new(:validate, \"test\")\n    iex> {{:emit, result}, _state} = handler.(signal, %{})\n    iex> result.type == :validated\n    true","ref":"AgentForge.DynamicFlow.html#subflow/1-examples"},{"type":"module","title":"AgentForge.Config","doc":"Provides configuration parsing for declarative workflows.\nSupports loading workflows from YAML or JSON.","ref":"AgentForge.Config.html"},{"type":"function","title":"AgentForge.Config.load_from_file/1","doc":"Loads a workflow from a YAML or JSON file.\nReturns a flow function that can be executed.","ref":"AgentForge.Config.html#load_from_file/1"},{"type":"function","title":"Examples - AgentForge.Config.load_from_file/1","doc":"iex> flow = AgentForge.Config.load_from_file(\"test/fixtures/simple_workflow.yaml\")\n    iex> is_function(flow, 1)\n    true","ref":"AgentForge.Config.html#load_from_file/1-examples"},{"type":"function","title":"AgentForge.Config.load_from_string/1","doc":"Loads a workflow from a YAML or JSON string.\nReturns a flow function that can be executed.","ref":"AgentForge.Config.html#load_from_string/1"},{"type":"function","title":"Examples - AgentForge.Config.load_from_string/1","doc":"iex> config = \"\"\"\n    ...> flow:\n    ...>   - type: transform\n    ...>     fn: upcase\n    ...>   - type: branch\n    ...>     condition: \"String.length(data) > 5\"\n    ...>     then:\n    ...>       - type: tool\n    ...>         name: notify\n    ...>     else:\n    ...>       - type: tool\n    ...>         name: log\n    ...> \"\"\"\n    iex> flow = AgentForge.Config.load_from_string(config)\n    iex> is_function(flow, 1)\n    true","ref":"AgentForge.Config.html#load_from_string/1-examples"},{"type":"module","title":"AgentForge.Debug","doc":"Provides debug utilities for tracing and logging signals through the system.","ref":"AgentForge.Debug.html"},{"type":"function","title":"AgentForge.Debug.log_signal_processing/2","doc":"Logs a signal's journey through the system.","ref":"AgentForge.Debug.html#log_signal_processing/2"},{"type":"function","title":"AgentForge.Debug.trace_flow/2","doc":"Creates a debug flow that wraps each handler in debug logging.","ref":"AgentForge.Debug.html#trace_flow/2"},{"type":"function","title":"Examples - AgentForge.Debug.trace_flow/2","doc":"iex> handlers = [\n    ...>   fn signal, state -> {AgentForge.Signal.emit(:step1, signal.data), state} end,\n    ...>   fn signal, state -> {AgentForge.Signal.emit(:step2, signal.data), state} end\n    ...> ]\n    iex> debug_flow = AgentForge.Debug.trace_flow(\"test_flow\", handlers)\n    iex> is_list(debug_flow) and length(debug_flow) == length(handlers)\n    true","ref":"AgentForge.Debug.html#trace_flow/2-examples"},{"type":"function","title":"AgentForge.Debug.trace_handler/2","doc":"Creates a debug handler that logs signal processing.\nWraps an existing handler with debug logging.","ref":"AgentForge.Debug.html#trace_handler/2"},{"type":"function","title":"Examples - AgentForge.Debug.trace_handler/2","doc":"iex> handler = fn signal, state -> {AgentForge.Signal.emit(:done, signal.data), state} end\n    iex> debug_handler = AgentForge.Debug.trace_handler(\"test\", handler)\n    iex> signal = AgentForge.Signal.new(:test, \"data\")\n    iex> {result, _state} = debug_handler.(signal, %{})\n    iex> match?({:emit, %{type: :done}}, result)\n    true","ref":"AgentForge.Debug.html#trace_handler/2-examples"},{"type":"module","title":"AgentForge.Runtime","doc":"Provides the runtime environment for executing flows in the AgentForge system.","ref":"AgentForge.Runtime.html"},{"type":"function","title":"AgentForge.Runtime.configure/2","doc":"Creates a new runtime configuration for a flow.","ref":"AgentForge.Runtime.html#configure/2"},{"type":"function","title":"AgentForge.Runtime.configure_stateful/2","doc":"Creates a new runtime configuration that maintains state between executions.","ref":"AgentForge.Runtime.html#configure_stateful/2"},{"type":"function","title":"AgentForge.Runtime.execute/3","doc":"Executes a flow with the given signal and options.","ref":"AgentForge.Runtime.html#execute/3"},{"type":"function","title":"AgentForge.Runtime.execute_with_limits/3","doc":"Executes a flow with execution limits.\n\nEnforces limits on execution (maximum steps and timeout) to prevent\ninfinite loops and long-running processes. Integrates with Store for\nstate persistence between executions.","ref":"AgentForge.Runtime.html#execute_with_limits/3"},{"type":"function","title":"Options - AgentForge.Runtime.execute_with_limits/3","doc":"* `:timeout_ms` - Maximum execution time in milliseconds (default: 30000)\n  * `:collect_stats` - Whether to collect execution statistics (default: true)\n  * `:return_stats` - Whether to include stats in the return value (default: false)\n  * `:debug` - Whether to enable debugging (default: false)\n  * `:name` - Name for debugging output (default: \"flow\")\n  * `:store_prefix` - Prefix for store keys (default: \"flow\")\n  * `:store_name` - Name of the store to use\n  * `:store_key` - Key within the store to access state\n  * `:initial_state` - Initial state to use for execution","ref":"AgentForge.Runtime.html#execute_with_limits/3-options"},{"type":"function","title":"AgentForge.Runtime.get_last_execution_stats/0","doc":"Gets statistics from the last flow execution.","ref":"AgentForge.Runtime.html#get_last_execution_stats/0"},{"type":"type","title":"AgentForge.Runtime.runtime_options/0","doc":"","ref":"AgentForge.Runtime.html#t:runtime_options/0"},{"type":"module","title":"AgentForge.Tools","doc":"Manages a registry of tools that can be used in workflows.\nEach tool is a function that can be executed with input data and returns a result.","ref":"AgentForge.Tools.html"},{"type":"function","title":"AgentForge.Tools.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"AgentForge.Tools.html#child_spec/1"},{"type":"function","title":"AgentForge.Tools.execute/2","doc":"Creates a function that executes a registered tool.","ref":"AgentForge.Tools.html#execute/2"},{"type":"function","title":"AgentForge.Tools.execute_pipeline/2","doc":"Creates a function that executes multiple tools in sequence.","ref":"AgentForge.Tools.html#execute_pipeline/2"},{"type":"function","title":"AgentForge.Tools.get/2","doc":"Retrieves a tool by name.","ref":"AgentForge.Tools.html#get/2"},{"type":"function","title":"AgentForge.Tools.list/1","doc":"Lists all registered tools.","ref":"AgentForge.Tools.html#list/1"},{"type":"function","title":"AgentForge.Tools.register/3","doc":"Registers a tool function with a given name.","ref":"AgentForge.Tools.html#register/3"},{"type":"function","title":"AgentForge.Tools.start_link/1","doc":"","ref":"AgentForge.Tools.html#start_link/1"},{"type":"function","title":"AgentForge.Tools.unregister/2","doc":"Unregisters a tool by name.","ref":"AgentForge.Tools.html#unregister/2"},{"type":"extras","title":"AgentForge","doc":"# AgentForge\n\n[![CI](https://github.com/i365dev/agent_forge/actions/workflows/ci.yml/badge.svg)](https://github.com/i365dev/agent_forge/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/agent_forge.svg)](https://hex.pm/packages/agent_forge)\n[![Docs](https://img.shields.io/badge/hex-docs-blue.svg)](https://hexdocs.pm/agent_forge)\n[![License](https://img.shields.io/badge/license-MIT-blue)](https://github.com/i365dev/agent_forge/blob/main/LICENSE)\n\nAgentForge is a lightweight, signal-driven workflow framework for Elixir, designed for building flexible and maintainable data processing pipelines.\n\n```mermaid\ngraph TB\n    Signal[Signal] --> Handler[Handler]\n    Handler --> Store[Store]\n    Handler --> Flow[Flow]\n    Flow --> Runtime[Runtime]\n```","ref":"readme.html"},{"type":"extras","title":"Features - AgentForge","doc":"- 🔄 **Signal-driven Architecture**: Build workflows around immutable signals\n- 🧩 **Composable Primitives**: Core building blocks for common patterns\n- 🔀 **Flexible Flows**: Chain handlers into dynamic processing pipelines\n- 📦 **State Management**: Track and update workflow state\n- ⚡ **Async Support**: Handle asynchronous operations\n- 🛠 **Configuration-based**: Define workflows in YAML\n- 💪 **Type-safe**: Leverages Elixir's pattern matching\n- 🔌 **Plugin System**: Extend functionality with custom plugins","ref":"readme.html#features"},{"type":"extras","title":"Quick Start - AgentForge","doc":"```elixir\n# Add to mix.exs\ndef deps do\n  [\n    {:agent_forge, \"~> 0.2.1\"}\n  ]\nend\n\n# Run\nmix deps.get\n```\n\n> **Note**: AgentForge 0.2.1 introduces the plugin system with improved code quality! See the [Plugin System Guide](guides/plugin_system.md) for more details.","ref":"readme.html#quick-start"},{"type":"extras","title":"Simple Example - AgentForge","doc":"```elixir\ndefmodule Example do\n  alias AgentForge.{Flow, Signal, Primitives}\n\n  def run do\n    # Define workflow steps\n    validate = Primitives.transform(fn data ->\n      if data.valid?, do: data, else: raise \"Invalid data\"\n    end)\n\n    process = Primitives.transform(fn data ->\n      Map.put(data, :processed, true)\n    end)\n\n    notify = Primitives.notify(\n      [:console],\n      format: &(\"Processed: #{inspect(&1)}\")\n    )\n\n    # Compose workflow\n    workflow = [validate, process, notify]\n\n    # Execute\n    signal = Signal.new(:start, %{valid?: true})\n    {:ok, result, _state} = Flow.process(workflow, signal, %{})\n    IO.inspect(result)\n  end\nend\n```","ref":"readme.html#simple-example"},{"type":"extras","title":"Core Components - AgentForge","doc":"","ref":"readme.html#core-components"},{"type":"extras","title":"Signals - AgentForge","doc":"Immutable messages that flow through the system:\n```elixir\nsignal = Signal.new(:user_action, %{id: 1})\n```","ref":"readme.html#signals"},{"type":"extras","title":"Primitives - AgentForge","doc":"Building blocks for common patterns:\n- **Branch**: Conditional processing\n- **Transform**: Data modification\n- **Loop**: Iteration handling\n- **Wait**: Async operations\n- **Notify**: Event notifications","ref":"readme.html#primitives"},{"type":"extras","title":"Flows - AgentForge","doc":"Compose handlers into pipelines:\n```elixir\nworkflow = [&validate/2, &process/2, &notify/2]\n```","ref":"readme.html#flows"},{"type":"extras","title":"Execution Limits - AgentForge","doc":"AgentForge now supports execution limits for flows to prevent long-running processes:\n\n```elixir\n# Create a handler\nhandler = fn signal, state ->\n  # Processing logic...\n  {{:emit, Signal.new(:done, result)}, state}\nend\n\n# Apply timeout limit\n{:ok, result, state} = AgentForge.process_with_limits(\n  [handler], \n  signal, \n  %{}, \n  timeout_ms: 5000  # Execution limited to 5 seconds\n)\n\n# Get execution statistics in the result\n{:ok, result, state, stats} = AgentForge.process_with_limits(\n  [handler], \n  signal, \n  %{}, \n  return_stats: true\n)\n\n# Or retrieve the last execution statistics afterwards\nstats = AgentForge.get_last_execution_stats()\n```\n\nThe execution limits feature supports the following options:\n- `timeout_ms`: Maximum execution time in milliseconds (default: `30000`)\n- `collect_stats`: Whether to collect execution statistics (default: `true`)\n- `return_stats`: Whether to include statistics in the return value (default: `false`)\n\nSee the documentation for more details.","ref":"readme.html#execution-limits"},{"type":"extras","title":"Documentation - AgentForge","doc":"- [AgentForge Design Philosophy & Architecture Guide](guides/design_guide.md)\n- [Getting Started Guide](guides/getting_started.md)\n- [Core Concepts](guides/core_concepts.md)\n- [Contribution Guidelines](CONTRIBUTING.md)","ref":"readme.html#documentation"},{"type":"extras","title":"Examples - AgentForge","doc":"- [Data Processing](examples/data_processing.exs): Basic data transformation pipeline\n- [Async Workflow](examples/async_workflow.exs): Handling async operations\n- [Configuration-based](examples/config_workflow.exs): YAML-defined workflows","ref":"readme.html#examples"},{"type":"extras","title":"Design Philosophy - AgentForge","doc":"AgentForge focuses on:\n- **Simplicity**: Clean, understandable codebase\n- **Flexibility**: Adaptable to various use cases\n- **Maintainability**: Well-documented, tested code\n- **Composability**: Build complex flows from simple parts","ref":"readme.html#design-philosophy"},{"type":"extras","title":"Use Cases - AgentForge","doc":"- ✅ Data Processing Pipelines\n- ✅ Event-driven Workflows\n- ✅ Multi-step Validations\n- ✅ Async Task Orchestration\n- ✅ Business Process Automation","ref":"readme.html#use-cases"},{"type":"extras","title":"Development - AgentForge","doc":"","ref":"readme.html#development"},{"type":"extras","title":"Setup - AgentForge","doc":"```bash\n# Clone the repository\ngit clone https://github.com/USERNAME/agent_forge.git\ncd agent_forge\n\n# Get dependencies\nmix deps.get\n\n# Run tests\nmix test\n\n# Generate documentation\nmix docs\n```","ref":"readme.html#setup"},{"type":"extras","title":"Pre-release Checklist - AgentForge","doc":"- [ ] All tests pass (`mix test`)\n- [ ] Test coverage is acceptable (`mix coveralls.html` - check coverage/excoveralls.html)\n- [ ] Code is formatted (`mix format`)\n- [ ] Documentation generates without errors (`mix docs`)\n- [ ] Version number is correct in mix.exs\n- [ ] CHANGELOG.md is updated\n- [ ] GitHub Actions workflows are in place\n- [ ] All examples run without errors","ref":"readme.html#pre-release-checklist"},{"type":"extras","title":"Contributing - AgentForge","doc":"We welcome contributions! Please see our [Contributing Guide](CONTRIBUTING.md) for guidelines.\n\n1. Read the [AgentForge Design Philosophy & Architecture Guide](guides/design_guide.md)\n2. Fork the repository\n3. Create your feature branch (`git checkout -b feature/amazing-feature`)\n4. Run the tests (`mix test`)\n5. Commit your changes (`git commit -m 'Add some amazing feature'`)\n6. Push to the branch (`git push origin feature/amazing-feature`)\n7. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - AgentForge","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to AgentForge will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]\n\n## [0.2.1] - 2025-03-27","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- Improved code quality and readability throughout the codebase\n- Reduced nested code in flow.ex, runtime.ex, primitives.ex, and tools.ex\n- Converted explicit try blocks to implicit try patterns\n- Removed trailing whitespace throughout codebase\n- Extracted helper functions to improve readability and reduce complexity\n\n## [0.2.0] - 2025-03-26","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Plugin system with formal plugin behavior interface\n- Central plugin manager for plugin lifecycle management\n- Tools registry for dynamic tool discovery and registration\n- Notification system with channel support\n- Built-in HTTP plugin as reference implementation\n- Comprehensive example (WeatherPlugin) demonstrating plugin functionality\n- Documentation for the plugin system\n\n## [0.1.1] - 2025-03-26","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Implement Time-Based Flow Limits for Signal Processing\n- Add Execution Limits and Statistics Tracking\n\n## [0.1.0] - 2025-03-23","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Initial release\n- Core signal-driven framework\n- Store implementation for state management\n- Flow composition utilities\n- Primitive system implementation\n  - Branch primitive\n  - Transform primitive\n  - Loop primitive\n  - Sequence primitive\n  - Wait primitive\n  - Notify primitive\n- Configuration-based workflow definition (YAML/JSON)\n- Dynamic flow selection and composition\n- Tool registry for extensible command execution\n- Comprehensive documentation and examples\n\n[Unreleased]: https://github.com/USERNAME/agent_forge/compare/v0.1.0...HEAD\n[0.1.0]: https://github.com/USERNAME/agent_forge/releases/tag/v0.1.0","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 i365 Tech\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Configuring AgentForge Workflows","doc":"# Configuring AgentForge Workflows\n\nThis guide explains how to define workflows using configuration files.","ref":"configuration.html"},{"type":"extras","title":"YAML Configuration - Configuring AgentForge Workflows","doc":"AgentForge supports defining workflows in YAML format:\n\n```yaml\nname: order_processing\ndescription: Handles incoming orders\n\nsteps:\n  - name: validate_order\n    type: transform\n    config:\n      validate:\n        - field: order_id\n          required: true\n        - field: amount\n          type: number\n          min: 0\n\n  - name: enrich_order\n    type: transform\n    config:\n      add_fields:\n        - timestamp: now()\n        - status: pending\n\n  - name: route_order\n    type: branch\n    config:\n      condition: \"amount >= 1000\"\n      then_flow: high_value_flow\n      else_flow: standard_flow\n\nflows:\n  high_value_flow:\n    - name: review_order\n      type: notify\n      config:\n        channels: [console, webhook]\n        message: \"High-value order {order_id} requires review\"\n\n  standard_flow:\n    - name: auto_process\n      type: transform\n      config:\n        process:\n          status: approved\n```","ref":"configuration.html#yaml-configuration"},{"type":"extras","title":"Configuration Structure - Configuring AgentForge Workflows","doc":"","ref":"configuration.html#configuration-structure"},{"type":"extras","title":"1. Workflow Metadata - Configuring AgentForge Workflows","doc":"```yaml\nname: workflow_name           # Required: Unique workflow identifier\ndescription: workflow_desc    # Optional: Description of the workflow\nversion: 1.0                 # Optional: Workflow version\n```","ref":"configuration.html#1-workflow-metadata"},{"type":"extras","title":"2. Steps Configuration - Configuring AgentForge Workflows","doc":"Each step must define:\n- `name`: Unique identifier within the workflow\n- `type`: Primitive type to use\n- `config`: Type-specific configuration","ref":"configuration.html#2-steps-configuration"},{"type":"extras","title":"3. Available Step Types - Configuring AgentForge Workflows","doc":"#### Transform\n```yaml\n- name: step_name\n  type: transform\n  config:\n    validate:              # Field validation\n      - field: field_name\n        required: true\n        type: string\n    add_fields:           # Add new fields\n      - field_name: value\n    remove_fields:        # Remove fields\n      - field_name\n```\n\n#### Branch\n```yaml\n- name: step_name\n  type: branch\n  config:\n    condition: \"expression\"    # Condition to evaluate\n    then_flow: flow_name      # Flow to use if true\n    else_flow: flow_name      # Flow to use if false\n```\n\n#### Loop\n```yaml\n- name: step_name\n  type: loop\n  config:\n    iterator: \"items\"         # Field containing items\n    item_handler: flow_name   # Flow to process each item\n```\n\n#### Wait\n```yaml\n- name: step_name\n  type: wait\n  config:\n    condition: \"expression\"   # Wait condition\n    timeout: 5000            # Timeout in milliseconds\n    retry_interval: 100      # Retry interval\n```\n\n#### Notify\n```yaml\n- name: step_name\n  type: notify\n  config:\n    channels: [console, webhook]\n    message: \"Template {variable}\"\n    format: \"json\"           # Optional format\n```","ref":"configuration.html#3-available-step-types"},{"type":"extras","title":"Using Configuration Files - Configuring AgentForge Workflows","doc":"","ref":"configuration.html#using-configuration-files"},{"type":"extras","title":"1. Loading Configuration - Configuring AgentForge Workflows","doc":"```elixir\ndefmodule MyWorkflow do\n  alias AgentForge.{Config, Flow}\n\n  def run(data) do\n    # Load workflow configuration\n    {:ok, workflow} = Config.load_file(\"workflows/order_processing.yaml\")\n    \n    # Create initial signal\n    signal = Signal.new(:order, data)\n    \n    # Execute workflow\n    Flow.process(workflow.steps, signal, %{})\n  end\nend\n```","ref":"configuration.html#1-loading-configuration"},{"type":"extras","title":"2. Dynamic Configuration - Configuring AgentForge Workflows","doc":"You can modify configuration at runtime:\n\n```elixir\ndef run(data, opts) do\n  {:ok, workflow} = Config.load_file(\"workflows/base.yaml\")\n  \n  # Modify configuration\n  workflow = Config.update_in(workflow, [:steps, :notify, :config], fn config ->\n    Map.put(config, :channels, opts.channels)\n  end)\n  \n  Flow.process(workflow.steps, signal, %{})\nend\n```","ref":"configuration.html#2-dynamic-configuration"},{"type":"extras","title":"Configuration Best Practices - Configuring AgentForge Workflows","doc":"1. **Organization**\n   - Group related workflows in directories\n   - Use clear, descriptive names\n   - Include version information\n\n2. **Reusability**\n   - Define common flows separately\n   - Use templates for repeated patterns\n   - Share configurations across similar workflows\n\n3. **Maintainability**\n   - Document configuration files\n   - Use consistent naming conventions\n   - Keep configurations focused\n\n4. **Validation**\n   - Validate configurations at load time\n   - Include schema definitions\n   - Test configuration variations","ref":"configuration.html#configuration-best-practices"},{"type":"extras","title":"Example Configurations - Configuring AgentForge Workflows","doc":"","ref":"configuration.html#example-configurations"},{"type":"extras","title":"1. Data Validation - Configuring AgentForge Workflows","doc":"```yaml\nname: data_validation\nsteps:\n  - name: validate_user\n    type: transform\n    config:\n      validate:\n        - field: email\n          required: true\n          pattern: \"^[^@]+@[^@]+$\"\n        - field: age\n          type: number\n          min: 18\n```","ref":"configuration.html#1-data-validation"},{"type":"extras","title":"2. Async Processing - Configuring AgentForge Workflows","doc":"```yaml\nname: async_process\nsteps:\n  - name: start_job\n    type: transform\n    config:\n      trigger_async: true\n  - name: wait_completion\n    type: wait\n    config:\n      condition: \"job_completed\"\n      timeout: 30000\n```","ref":"configuration.html#2-async-processing"},{"type":"extras","title":"3. Conditional Flows - Configuring AgentForge Workflows","doc":"```yaml\nname: conditional_process\nsteps:\n  - name: check_data\n    type: branch\n    config:\n      condition: \"valid_data\"\n      then_flow: process_flow\n      else_flow: error_flow\n```","ref":"configuration.html#3-conditional-flows"},{"type":"extras","title":"Debugging Configuration - Configuring AgentForge Workflows","doc":"1. **Logging**\n```yaml\nname: debug_workflow\nsteps:\n  - name: debug_step\n    type: notify\n    config:\n      channels: [console]\n      message: \"Processing: {data}\"\n      log_level: debug\n```\n\n2. **Testing Configurations**\n```elixir\ndef test_config do\n  {:ok, workflow} = Config.load_file(\"test/fixtures/workflow.yaml\")\n  \n  # Validate configuration\n  assert workflow.name == \"test_workflow\"\n  assert length(workflow.steps) > 0\n  \n  # Test with sample data\n  signal = Signal.new(:test, sample_data)\n  {:ok, result, _} = Flow.process(workflow.steps, signal, %{})\nend\n```","ref":"configuration.html#debugging-configuration"},{"type":"extras","title":"Next Steps - Configuring AgentForge Workflows","doc":"1. Review example workflows in `examples/workflows/`\n2. Try modifying existing configurations\n3. Create custom configurations\n4. Learn about [extending primitives](extending_primitives.md)","ref":"configuration.html#next-steps"},{"type":"extras","title":"AgentForge Core Concepts","doc":"# AgentForge Core Concepts\n\nThis guide explains the fundamental concepts of the AgentForge framework and how they work together to create dynamic workflows.","ref":"core_concepts.html"},{"type":"extras","title":"Core Components - AgentForge Core Concepts","doc":"","ref":"core_concepts.html#core-components"},{"type":"extras","title":"1. Signals - AgentForge Core Concepts","doc":"Signals are the fundamental unit of communication in AgentForge. They carry both data and metadata about events or actions in the system.\n\n```elixir\n# Basic signal creation\nsignal = Signal.new(:user_action, %{id: 1, data: \"example\"})\n\n# Signal structure\n%{\n  type: :user_action,\n  data: %{id: 1, data: \"example\"},\n  meta: %{timestamp: ~U[2025-03-22 10:00:00Z]}\n}\n```","ref":"core_concepts.html#1-signals"},{"type":"extras","title":"2. Handlers - AgentForge Core Concepts","doc":"Handlers are functions that process signals and maintain state. They follow a consistent interface:\n\n```elixir\n# Handler function signature\n(signal, state) -> {result, new_state}\n\n# Example handler\ndef process_user(signal, state) do\n  new_state = Map.put(state, :last_user, signal.data)\n  {Signal.emit(:user_processed, signal.data), new_state}\nend\n```","ref":"core_concepts.html#2-handlers"},{"type":"extras","title":"3. Flows - AgentForge Core Concepts","doc":"Flows compose multiple handlers into processing pipelines. They manage the execution order and error handling.\n\n```elixir\n# Creating a flow\nflow = [\n  &validate_user/2,\n  &enrich_user_data/2,\n  &save_user/2\n]\n\n# Processing a signal through the flow\n{:ok, result, final_state} = Flow.process(flow, signal, initial_state)\n```","ref":"core_concepts.html#3-flows"},{"type":"extras","title":"4. Primitives - AgentForge Core Concepts","doc":"Primitives are building blocks for creating handlers. AgentForge provides several core primitives:\n\n#### Branch Primitive\nConditionally executes different flows based on a condition:\n\n```elixir\nbranch = Primitives.branch(\n  fn signal, _ -> signal.data.age >= 18 end,\n  adult_flow,\n  minor_flow\n)\n```\n\n#### Transform Primitive\nModifies signal data:\n\n```elixir\ntransform = Primitives.transform(fn data ->\n  Map.put(data, :processed_at, DateTime.utc_now())\nend)\n```\n\n#### Loop Primitive\nIterates over items in signal data:\n\n```elixir\nloop = Primitives.loop(fn item, state ->\n  {Signal.emit(:item_processed, item), state}\nend)\n```\n\n#### Wait Primitive\nPauses execution until a condition is met:\n\n```elixir\nwait = Primitives.wait(\n  fn _, state -> state.resource_ready end,\n  timeout: 5000\n)\n```\n\n#### Notify Primitive\nSends notifications through configured channels:\n\n```elixir\nnotify = Primitives.notify(\n  [:console, :webhook],\n  format: &(\"User #{&1.name} registered\")\n)\n```","ref":"core_concepts.html#4-primitives"},{"type":"extras","title":"5. State Management - AgentForge Core Concepts","doc":"State is maintained throughout the workflow execution:\n\n```elixir\n# Initial state\nstate = %{counter: 0}\n\n# Handler updating state\ndef count_signal(signal, state) do\n  new_state = Map.update(state, :counter, 1, & &1 + 1)\n  {signal, new_state}\nend\n```","ref":"core_concepts.html#5-state-management"},{"type":"extras","title":"Common Patterns - AgentForge Core Concepts","doc":"","ref":"core_concepts.html#common-patterns"},{"type":"extras","title":"1. Signal Chaining - AgentForge Core Concepts","doc":"```elixir\ndef process_order(signal, state) do\n  with {:ok, validated} <- validate_order(signal.data),\n       {:ok, enriched} <- enrich_order(validated),\n       {:ok, saved} <- save_order(enriched) do\n    {Signal.emit(:order_processed, saved), state}\n  else\n    {:error, reason} -> {Signal.emit(:order_failed, reason), state}\n  end\nend\n```","ref":"core_concepts.html#1-signal-chaining"},{"type":"extras","title":"2. State Accumulation - AgentForge Core Concepts","doc":"```elixir\ndef aggregate_totals(signal, state) do\n  total = Map.get(state, :total, 0) + signal.data.amount\n  {signal, Map.put(state, :total, total)}\nend\n```","ref":"core_concepts.html#2-state-accumulation"},{"type":"extras","title":"3. Conditional Processing - AgentForge Core Concepts","doc":"```elixir\ndef route_request(signal, state) do\n  case signal.data.priority do\n    :high -> {Signal.emit(:urgent, signal.data), state}\n    :low -> {Signal.emit(:routine, signal.data), state}\n  end\nend\n```","ref":"core_concepts.html#3-conditional-processing"},{"type":"extras","title":"Configuration-based Workflows - AgentForge Core Concepts","doc":"AgentForge supports defining workflows through configuration:\n\n```yaml\nname: user_registration\nsteps:\n  - name: validate_input\n    type: transform\n    config:\n      validate:\n        - field: email\n          required: true\n  - name: process_user\n    type: branch\n    config:\n      condition: \"age >= 18\"\n      then_flow: adult_flow\n      else_flow: minor_flow\n```","ref":"core_concepts.html#configuration-based-workflows"},{"type":"extras","title":"Error Handling - AgentForge Core Concepts","doc":"AgentForge provides several ways to handle errors:\n\n1. Return tagged tuples:\n```elixir\n{:error, reason} -> {Signal.emit(:error, reason), state}\n```\n\n2. Use rescue in transforms:\n```elixir\nPrimitives.transform(fn data ->\n  # ... risky operation ...\nrescue\n  e -> raise \"Processing failed: #{Exception.message(e)}\"\nend)\n```\n\n3. Handle errors in flows:\n```elixir\ncase Flow.process(workflow, signal, state) do\n  {:ok, result, final_state} -> handle_success(result)\n  {:error, reason} -> handle_error(reason)\nend\n```","ref":"core_concepts.html#error-handling"},{"type":"extras","title":"Best Practices - AgentForge Core Concepts","doc":"1. Keep handlers small and focused\n2. Use appropriate primitives for common patterns\n3. Maintain immutable state\n4. Handle errors at appropriate levels\n5. Use clear signal types and meaningful data\n6. Document handlers and flows\n7. Test different execution paths","ref":"core_concepts.html#best-practices"},{"type":"extras","title":"Next Steps - AgentForge Core Concepts","doc":"- Check out the examples in the `examples/` directory\n- Read the primitive-specific guides\n- Review the test files for more usage patterns\n- See the CONTRIBUTING.md file for development guidelines","ref":"core_concepts.html#next-steps"},{"type":"extras","title":"AgentForge Design Philosophy & Architecture Guide","doc":"# AgentForge Design Philosophy & Architecture Guide","ref":"design_guide.html"},{"type":"extras","title":"Introduction - AgentForge Design Philosophy & Architecture Guide","doc":"AgentForge is a lightweight, signal-driven workflow framework designed for building flexible and maintainable data processing pipelines. This guide articulates AgentForge's architectural focus, design principles, and recommendations for extending its functionality.","ref":"design_guide.html#introduction"},{"type":"extras","title":"Core Focus & Design Philosophy - AgentForge Design Philosophy & Architecture Guide","doc":"AgentForge is built on the following key principles:\n\n1. **Simplicity First**: Maintain a minimal, clean codebase that's easy to understand and extend\n2. **Signal-Driven Architecture**: Base all operations on immutable signals passing through handler chains\n3. **Composable Primitives**: Provide fundamental building blocks that can be combined into complex workflows\n4. **State Management**: Support clean, predictable state transitions throughout workflow execution\n5. **Execution Control**: Provide mechanisms to control and monitor workflow execution\n\nThe framework is designed to excel at workflow orchestration - the coordination, sequencing, and conditional execution of processing steps - while intentionally delegating certain responsibilities to the application layer or specialized libraries.","ref":"design_guide.html#core-focus-design-philosophy"},{"type":"extras","title":"What Belongs in the Core Framework - AgentForge Design Philosophy & Architecture Guide","doc":"AgentForge's core should remain focused on these fundamental capabilities:","ref":"design_guide.html#what-belongs-in-the-core-framework"},{"type":"extras","title":"1. Signal Processing - AgentForge Design Philosophy & Architecture Guide","doc":"- Signal creation, transformation, and emission\n- Metadata handling and correlation\n- Type-based routing","ref":"design_guide.html#1-signal-processing"},{"type":"extras","title":"2. Flow Composition - AgentForge Design Philosophy & Architecture Guide","doc":"- Handler chaining and execution\n- Error propagation and handling\n- Execution statistics and monitoring","ref":"design_guide.html#2-flow-composition"},{"type":"extras","title":"3. Core Primitives - AgentForge Design Philosophy & Architecture Guide","doc":"- Branch: Conditional execution\n- Transform: Data modification\n- Loop: Iterative processing\n- Sequence: Ordered execution\n- Notify (basic): Simple notification outputs\n- Wait: Condition-based pausing","ref":"design_guide.html#3-core-primitives"},{"type":"extras","title":"4. Configuration Loading - AgentForge Design Philosophy & Architecture Guide","doc":"- Configuration-driven workflow definition\n- YAML/JSON parsing and validation\n- Dynamic flow generation","ref":"design_guide.html#4-configuration-loading"},{"type":"extras","title":"5. Execution Control - AgentForge Design Philosophy & Architecture Guide","doc":"- Timeout limitations\n- Execution statistics\n- Runtime state inspection","ref":"design_guide.html#5-execution-control"},{"type":"extras","title":"What Belongs at the Application Layer - AgentForge Design Philosophy & Architecture Guide","doc":"The following concerns are better implemented at the application layer:","ref":"design_guide.html#what-belongs-at-the-application-layer"},{"type":"extras","title":"1. Scheduling & Timing - AgentForge Design Philosophy & Architecture Guide","doc":"- Periodic task execution\n- Cron-style scheduling\n- Job queuing and prioritization\n\n**Rationale**: Scheduling systems require complex state persistence, error recovery, and distributed coordination that would significantly increase framework complexity. Elixir's ecosystem already provides excellent solutions like Quantum and Oban.","ref":"design_guide.html#1-scheduling-timing"},{"type":"extras","title":"2. Data Source Integration - AgentForge Design Philosophy & Architecture Guide","doc":"- Database connectors\n- API clients\n- File system operations\n- Streaming data sources\n\n**Rationale**: Different applications have vastly different data source requirements. Embedding specific connectors would add unnecessary dependencies and maintenance burden.","ref":"design_guide.html#2-data-source-integration"},{"type":"extras","title":"3. External Service Integration - AgentForge Design Philosophy & Architecture Guide","doc":"- LLM API clients\n- Authentication handling\n- Rate limiting and retries\n- Service-specific error handling\n\n**Rationale**: These integrations evolve rapidly, have diverse configuration requirements, and are typically specific to particular use cases.","ref":"design_guide.html#3-external-service-integration"},{"type":"extras","title":"4. Advanced Notification Systems - AgentForge Design Philosophy & Architecture Guide","doc":"- Email delivery\n- SMS messaging\n- Chat platform integration\n- Notification templating\n\n**Rationale**: While basic notification is a core primitive, delivery mechanisms involve complex dependencies and configuration that vary significantly between applications.","ref":"design_guide.html#4-advanced-notification-systems"},{"type":"extras","title":"Integration Patterns - AgentForge Design Philosophy & Architecture Guide","doc":"To integrate these application-layer concerns with AgentForge, we recommend the following patterns:","ref":"design_guide.html#integration-patterns"},{"type":"extras","title":"1. Tool-Based Integration - AgentForge Design Philosophy & Architecture Guide","doc":"Register external functionality as tools:\n\n```elixir\n# Register an HTTP client tool\nTools.register(\"http_get\", fn config ->\n  HTTPoison.get!(config[\"url\"], config[\"headers\"] || [])\n  |> Map.get(:body)\n  |> Jason.decode!()\nend)\n\n# Register an LLM tool\nTools.register(\"analyze_with_llm\", fn input ->\n  OpenAI.chat_completion(\n    model: \"gpt-4\",\n    messages: [%{role: \"user\", content: \"Analyze: #{input}\"}]\n  )\n  |> Map.get(:choices)\n  |> List.first()\n  |> Map.get(:message)\n  |> Map.get(:content)\nend)\n```","ref":"design_guide.html#1-tool-based-integration"},{"type":"extras","title":"2. Configuration-Driven Integration - AgentForge Design Philosophy & Architecture Guide","doc":"Define external integrations in workflow configuration:\n\n```yaml\n# workflow.yaml\nflow:\n  - type: tool\n    name: http_get\n    config:\n      url: \"https://api.example.com/data\"\n  - type: transform\n    fn: \"process_data\"\n  - type: tool\n    name: analyze_with_llm\n```","ref":"design_guide.html#2-configuration-driven-integration"},{"type":"extras","title":"3. Scheduler-Based Triggering - AgentForge Design Philosophy & Architecture Guide","doc":"Use external schedulers to trigger AgentForge workflows:\n\n```elixir\n# In application code\nconfig :your_app, YourApp.Scheduler,\n  jobs: [\n    {\"0 */1 * * *\", fn -> \n      signal = Signal.new(:scheduled_run, %{timestamp: DateTime.utc_now()})\n      workflow = load_workflow_from_config(\"workflows/market_analysis.yaml\")\n      AgentForge.process_with_limits(workflow, signal, %{}, timeout_ms: 60000)\n    end}\n  ]\n```","ref":"design_guide.html#3-scheduler-based-triggering"},{"type":"extras","title":"Best Practices - AgentForge Design Philosophy & Architecture Guide","doc":"","ref":"design_guide.html#best-practices"},{"type":"extras","title":"1. Keep the Core Framework Thin - AgentForge Design Philosophy & Architecture Guide","doc":"- Focus on the essential orchestration capabilities\n- Resist adding dependencies for specialized functionality\n- Prioritize extensibility over built-in features","ref":"design_guide.html#1-keep-the-core-framework-thin"},{"type":"extras","title":"2. Use Composition Over Configuration - AgentForge Design Philosophy & Architecture Guide","doc":"- Compose small, focused handlers rather than complex, configurable ones\n- Allow behavior to emerge from composition rather than parameterization\n- Build reusable handler libraries at the application level","ref":"design_guide.html#2-use-composition-over-configuration"},{"type":"extras","title":"3. Develop Standard Patterns for Common Needs - AgentForge Design Philosophy & Architecture Guide","doc":"- Create and share reference implementations for common integration patterns\n- Document best practices for specific use cases\n- Provide examples rather than built-in solutions","ref":"design_guide.html#3-develop-standard-patterns-for-common-needs"},{"type":"extras","title":"4. Consider Extensibility Through Plugins - AgentForge Design Philosophy & Architecture Guide","doc":"For commonly needed functionality that doesn't belong in the core, consider a plugin architecture that allows:\n\n- Registration of custom primitives\n- Extension of existing primitives\n- Addition of new tool types\n- Configuration of framework behavior","ref":"design_guide.html#4-consider-extensibility-through-plugins"},{"type":"extras","title":"Conclusion - AgentForge Design Philosophy & Architecture Guide","doc":"AgentForge's strength lies in its focused approach to workflow orchestration. By maintaining a clear separation of concerns between the core framework and application-specific functionality, we ensure AgentForge remains lightweight, maintainable, and adaptable to diverse use cases.\n\nThe framework should provide the essential building blocks for workflow orchestration while empowering developers to integrate with specialized tools and services at the application layer, creating a powerful ecosystem without bloating the core framework.","ref":"design_guide.html#conclusion"},{"type":"extras","title":"Execution Limits","doc":"# Execution Limits\n\nAgentForge provides execution limits to ensure that your workflows behave predictably and efficiently. This guide covers the use of timeouts and execution statistics to monitor and control your flows.","ref":"execution_limits.html"},{"type":"extras","title":"Table of Contents - Execution Limits","doc":"- [Overview](#overview)\n- [Timeout Limits](#timeout-limits)\n- [Execution Statistics](#execution-statistics)\n- [Error Handling](#error-handling)\n- [API Reference](#api-reference)\n- [Examples](#examples)","ref":"execution_limits.html#table-of-contents"},{"type":"extras","title":"Overview - Execution Limits","doc":"When running complex workflows, especially those interacting with external systems or performing intensive computations, it's important to have safeguards against:\n\n- Infinite loops\n- Long-running operations\n- Resource exhaustion\n- Unresponsive services\n\nAgentForge's execution limits provide these safeguards through timeouts and detailed statistics tracking.","ref":"execution_limits.html#overview"},{"type":"extras","title":"Timeout Limits - Execution Limits","doc":"","ref":"execution_limits.html#timeout-limits"},{"type":"extras","title":"Setting Timeouts - Execution Limits","doc":"You can set a timeout in milliseconds for any flow processing:\n\n```elixir\n# Create a handler\nhandler = fn signal, state ->\n  # Long-running operation...\n  {{:emit, Signal.new(:done, result)}, state}\nend\n\n# Apply a 5-second timeout\n{:ok, result, state} = AgentForge.process_with_limits(\n  [handler], \n  signal, \n  %{}, \n  timeout_ms: 5000  # 5 second timeout\n)\n```\n\nIf the processing exceeds the timeout, it will be terminated and return an error:\n\n```elixir\n{:error, \"Flow execution timed out after 5000ms\", state}\n```","ref":"execution_limits.html#setting-timeouts"},{"type":"extras","title":"Default Timeout - Execution Limits","doc":"If not specified, the default timeout is 30 seconds (30,000 ms). You can adjust this based on your application's needs.","ref":"execution_limits.html#default-timeout"},{"type":"extras","title":"Execution Statistics - Execution Limits","doc":"AgentForge can collect detailed statistics about flow execution, including:\n\n- Number of steps executed\n- Total execution time\n- Completion status\n- Result of execution","ref":"execution_limits.html#execution-statistics"},{"type":"extras","title":"Collecting Statistics - Execution Limits","doc":"Statistics are collected by default but not returned unless requested:\n\n```elixir\n# Get statistics in the result\n{:ok, result, state, stats} = AgentForge.process_with_limits(\n  handlers, \n  signal, \n  %{}, \n  return_stats: true\n)\n\n# Examine the statistics\nIO.inspect(stats.steps)      # Number of steps executed\nIO.inspect(stats.elapsed_ms) # Execution time in milliseconds\nIO.inspect(stats.complete)   # Whether execution completed normally\n```","ref":"execution_limits.html#collecting-statistics"},{"type":"extras","title":"Retrieving Last Execution Statistics - Execution Limits","doc":"Even if you don't request statistics in the return value, you can retrieve them afterward:\n\n```elixir\n# Process without requesting statistics in the return\n{:ok, result, state} = AgentForge.process_with_limits(handlers, signal, %{})\n\n# Retrieve statistics later\nstats = AgentForge.get_last_execution_stats()\n```\n\nThis is particularly useful for logging and monitoring.","ref":"execution_limits.html#retrieving-last-execution-statistics"},{"type":"extras","title":"Error Handling - Execution Limits","doc":"Execution limits can produce several error scenarios:","ref":"execution_limits.html#error-handling"},{"type":"extras","title":"Timeout Errors - Execution Limits","doc":"When a flow exceeds its time limit:\n\n```elixir\n{:error, \"Flow execution timed out after 5000ms\", state}\n```\n\nWith statistics:\n\n```elixir\n{:error, \"Flow execution timed out after 5000ms\", state, stats}\n```","ref":"execution_limits.html#timeout-errors"},{"type":"extras","title":"Handler Errors - Execution Limits","doc":"When a handler raises an exception:\n\n```elixir\n{:error, \"Flow processing error: ...\", state}\n```","ref":"execution_limits.html#handler-errors"},{"type":"extras","title":"Handling Errors Gracefully - Execution Limits","doc":"Always wrap flow execution in appropriate error handling:\n\n```elixir\ncase AgentForge.process_with_limits(handlers, signal, state, timeout_ms: 5000) do\n  {:ok, result, new_state} ->\n    # Process completed successfully\n    handle_success(result, new_state)\n    \n  {:error, \"Flow execution timed out\" <> _, state} ->\n    # Handle timeout specifically\n    handle_timeout(state)\n    \n  {:error, error_message, state} ->\n    # Handle other errors\n    handle_error(error_message, state)\nend\n```","ref":"execution_limits.html#handling-errors-gracefully"},{"type":"extras","title":"API Reference - Execution Limits","doc":"### `AgentForge.process_with_limits/4`\n\n```elixir\n@spec process_with_limits(\n  [handler_function], \n  Signal.t(), \n  state_map, \n  options\n) :: \n  {:ok, Signal.t(), state_map} | \n  {:ok, Signal.t(), state_map, ExecutionStats.t()} | \n  {:error, String.t(), state_map} | \n  {:error, String.t(), state_map, ExecutionStats.t()}\n```\n\nOptions:\n- `timeout_ms`: Maximum execution time in milliseconds (default: 30000)\n- `collect_stats`: Whether to collect execution statistics (default: true)\n- `return_stats`: Whether to include statistics in the return value (default: false)\n- `store_name`: Name of the store to use for state persistence\n- `store_key`: Key within the store to access state\n\n### `AgentForge.get_last_execution_stats/0`\n\n```elixir\n@spec get_last_execution_stats() :: ExecutionStats.t() | nil\n```\n\nReturns the statistics from the last flow execution or nil if none are available.","ref":"execution_limits.html#api-reference"},{"type":"extras","title":"Examples - Execution Limits","doc":"","ref":"execution_limits.html#examples"},{"type":"extras","title":"Basic Timeout Example - Execution Limits","doc":"```elixir\n# Define a handler that may take too long\npotentially_slow_handler = fn signal, state ->\n  result = perform_intensive_operation(signal.data)\n  {{:emit, Signal.new(:processed, result)}, state}\nend\n\n# Process with a timeout\ncase AgentForge.process_with_limits([potentially_slow_handler], signal, %{}, timeout_ms: 10000) do\n  {:ok, result, state} ->\n    IO.puts(\"Completed successfully: #{inspect(result.data)}\")\n    \n  {:error, error_message, _state} ->\n    IO.puts(\"Error: #{error_message}\")\nend\n```","ref":"execution_limits.html#basic-timeout-example"},{"type":"extras","title":"Collecting Performance Metrics - Execution Limits","doc":"```elixir\n# Process and collect statistics\n{:ok, result, _state, stats} = AgentForge.process_with_limits(\n  workflow, \n  signal, \n  %{}, \n  return_stats: true\n)\n\n# Log performance metrics\nLogger.info(\"Workflow completed in #{stats.elapsed_ms}ms with #{stats.steps} steps\")\n```\n\nFor a complete working example, see [limited_workflow.exs](../examples/limited_workflow.exs) in the examples directory.","ref":"execution_limits.html#collecting-performance-metrics"},{"type":"extras","title":"Extending AgentForge Primitives","doc":"# Extending AgentForge Primitives\n\nThis guide explains how to create custom primitives for AgentForge.","ref":"extending_primitives.html"},{"type":"extras","title":"Primitive Structure - Extending AgentForge Primitives","doc":"A primitive is a function that returns a handler function with the signature:\n```elixir\n(signal, state) -> {result, new_state}\n```","ref":"extending_primitives.html#primitive-structure"},{"type":"extras","title":"Creating Custom Primitives - Extending AgentForge Primitives","doc":"","ref":"extending_primitives.html#creating-custom-primitives"},{"type":"extras","title":"Basic Template - Extending AgentForge Primitives","doc":"```elixir\ndef my_primitive(options) do\n  fn signal, state ->\n    # Process signal and state\n    {{:emit, new_signal}, new_state}\n  end\nend\n```","ref":"extending_primitives.html#basic-template"},{"type":"extras","title":"Result Types - Extending AgentForge Primitives","doc":"Your primitive should return one of these result types:\n\n```elixir\n{{:emit, signal}, new_state}           # Emit single signal\n{{:emit_many, signals}, new_state}     # Emit multiple signals\n{{:halt, result}, new_state}           # Stop processing\n{{:wait, reason}, new_state}           # Pause processing\n{{:error, reason}, state}              # Error condition\n{:skip, new_state}                     # Skip further processing\n```","ref":"extending_primitives.html#result-types"},{"type":"extras","title":"Example: Rate Limiter Primitive - Extending AgentForge Primitives","doc":"Here's a complete example of a custom primitive that implements rate limiting:\n\n```elixir\ndefmodule MyPrimitives do\n  alias AgentForge.{Signal}\n\n  @doc \"\"\"\n  Creates a rate limiter primitive that controls signal processing rate.","ref":"extending_primitives.html#example-rate-limiter-primitive"},{"type":"extras","title":"Options - Extending AgentForge Primitives","doc":"* `:limit` - Maximum number of signals per interval\n  * `:interval` - Time interval in milliseconds","ref":"extending_primitives.html#options"},{"type":"extras","title":"Examples - Extending AgentForge Primitives","doc":"iex> limiter = rate_limit(limit: 2, interval: 1000)\n      iex> signal = Signal.new(:test, \"data\")\n      iex> {{:emit, result}, state} = limiter.(signal, %{})\n  \"\"\"\n  def rate_limit(opts) do\n    limit = Keyword.get(opts, :limit, 10)\n    interval = Keyword.get(opts, :interval, 1000)\n\n    fn signal, state ->\n      current_time = System.monotonic_time(:millisecond)\n      window_start = Map.get(state, :window_start, current_time)\n      count = Map.get(state, :count, 0)\n\n      cond do\n        # New window\n        current_time - window_start > interval ->\n          new_state = %{\n            window_start: current_time,\n            count: 1\n          }\n          {{:emit, signal}, new_state}\n\n        # Within limit\n        count  \n          new_state = %{\n            window_start: window_start,\n            count: count + 1\n          }\n          {{:emit, signal}, new_state}\n\n        # Rate limit exceeded\n        true ->\n          wait_time = window_start + interval - current_time\n          {{:wait, \"Rate limit exceeded, retry in #{wait_time}ms\"}, state}\n      end\n    end\n  end\nend\n```","ref":"extending_primitives.html#examples"},{"type":"extras","title":"Testing Custom Primitives - Extending AgentForge Primitives","doc":"Follow these testing patterns:\n\n```elixir\ndefmodule MyPrimitivesTest do\n  use ExUnit.Case\n  alias AgentForge.Signal\n\n  describe \"rate_limit/1\" do\n    test \"allows signals within limit\" do\n      limiter = MyPrimitives.rate_limit(limit: 2, interval: 1000)\n      signal = Signal.new(:test, \"data\")\n      state = %{}\n\n      # First signal\n      {{:emit, result1}, state1} = limiter.(signal, state)\n      assert result1.type == :test\n\n      # Second signal\n      {{:emit, result2}, _state2} = limiter.(signal, state1)\n      assert result2.type == :test\n    end\n\n    test \"rate limits excessive signals\" do\n      limiter = MyPrimitives.rate_limit(limit: 1, interval: 1000)\n      signal = Signal.new(:test, \"data\")\n      state = %{}\n\n      {{:emit, _}, state1} = limiter.(signal, state)\n      {{:wait, reason}, _} = limiter.(signal, state1)\n\n      assert reason =~ \"Rate limit exceeded\"\n    end\n  end\nend\n```","ref":"extending_primitives.html#testing-custom-primitives"},{"type":"extras","title":"Best Practices - Extending AgentForge Primitives","doc":"1. **Documentation**\n   - Use `@moduledoc` and `@doc` with examples\n   - Document all options\n   - Include usage patterns\n\n2. **Options Handling**\n   - Use `Keyword.get/3` for optional parameters\n   - Validate options early\n   - Provide sensible defaults\n\n3. **State Management**\n   - Keep state immutable\n   - Document state structure\n   - Clean up state when needed\n\n4. **Error Handling**\n   - Use clear error messages\n   - Return `{:error, reason}` for recoverable errors\n   - Raise only for programming errors\n\n5. **Testing**\n   - Test happy paths\n   - Test error conditions\n   - Test state transitions\n   - Test edge cases","ref":"extending_primitives.html#best-practices"},{"type":"extras","title":"Integration - Extending AgentForge Primitives","doc":"To use your custom primitives:\n\n```elixir\ndefmodule MyWorkflow do\n  alias AgentForge.{Flow, Signal}\n  alias MyPrimitives\n\n  def run do\n    # Create rate-limited workflow\n    workflow = [\n      MyPrimitives.rate_limit(limit: 10, interval: 1000),\n      &process_data/2\n    ]\n\n    # Execute\n    signal = Signal.new(:start, \"data\")\n    Flow.process(workflow, signal, %{})\n  end\n\n  defp process_data(signal, state) do\n    # ... processing logic ...\n  end\nend\n```","ref":"extending_primitives.html#integration"},{"type":"extras","title":"Advanced Patterns - Extending AgentForge Primitives","doc":"","ref":"extending_primitives.html#advanced-patterns"},{"type":"extras","title":"Composing Primitives - Extending AgentForge Primitives","doc":"```elixir\ndef rate_limited_transform(transform_fn, rate_opts) do\n  fn signal, state ->\n    limiter = rate_limit(rate_opts)\n    transformer = transform(transform_fn)\n\n    case limiter.(signal, state) do\n      {{:emit, limited_signal}, new_state} ->\n        transformer.(limited_signal, new_state)\n      other -> other\n    end\n  end\nend\n```","ref":"extending_primitives.html#composing-primitives"},{"type":"extras","title":"Stateful Primitives - Extending AgentForge Primitives","doc":"```elixir\ndef with_cache(ttl_ms) do\n  fn signal, state ->\n    cache = Map.get(state, :cache, %{})\n    key = \"#{signal.type}:#{inspect(signal.data)}\"\n    current_time = System.monotonic_time(:millisecond)\n\n    case cache do\n      %{^key => {value, expires_at}} when current_time  \n        {{:emit, Signal.new(:cached, value)}, state}\n      _ ->\n        # Process and cache\n        new_cache = Map.put(cache, key, {\n          signal.data,\n          current_time + ttl_ms\n        })\n        {{:emit, signal}, Map.put(state, :cache, new_cache)}\n    end\n  end\nend\n```","ref":"extending_primitives.html#stateful-primitives"},{"type":"extras","title":"Next Steps - Extending AgentForge Primitives","doc":"- Review core primitives source code for patterns\n- Contribute primitives back to the community\n- Share your implementations in discussions","ref":"extending_primitives.html#next-steps"},{"type":"extras","title":"Getting Started with AgentForge","doc":"# Getting Started with AgentForge\n\nThis guide will help you get up and running with AgentForge quickly.","ref":"getting_started.html"},{"type":"extras","title":"Installation - Getting Started with AgentForge","doc":"Add AgentForge to your mix.exs dependencies:\n\n```elixir\ndef deps do\n  [\n    {:agent_forge, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"getting_started.html#installation"},{"type":"extras","title":"Quick Start Example - Getting Started with AgentForge","doc":"Let's create a simple workflow that processes user registration:\n\n```elixir\ndefmodule UserRegistration do\n  alias AgentForge.{Flow, Signal, Primitives}\n\n  def run do\n    # 1. Define validation transform\n    validate = Primitives.transform(fn user ->\n      cond do\n        is_nil(user.email) -> raise \"Email is required\"\n        String.length(user.password) < 8 -> raise \"Password too short\"\n        true -> user\n      end\n    end)\n\n    # 2. Define data enrichment\n    enrich = Primitives.transform(fn user ->\n      Map.merge(user, %{\n        created_at: DateTime.utc_now(),\n        status: :active\n      })\n    end)\n\n    # 3. Define notification\n    notify = Primitives.notify(\n      [:console],\n      format: &(\"New user registered: #{&1.email}\")\n    )\n\n    # 4. Compose workflow\n    workflow = [validate, enrich, notify]\n\n    # 5. Process a registration\n    user = %{email: \"user@example.com\", password: \"secret123\"}\n    signal = Signal.new(:registration, user)\n    \n    case Flow.process(workflow, signal, %{}) do\n      {:ok, result, _state} ->\n        IO.puts(\"Registration successful!\")\n        IO.inspect(result)\n      \n      {:error, reason} ->\n        IO.puts(\"Registration failed: #{reason}\")\n    end\n  end\nend\n\n# Run the example\nUserRegistration.run()\n```","ref":"getting_started.html#quick-start-example"},{"type":"extras","title":"Core Concepts - Getting Started with AgentForge","doc":"","ref":"getting_started.html#core-concepts"},{"type":"extras","title":"1. Signals - Getting Started with AgentForge","doc":"Signals are messages that flow through your system:\n\n```elixir\n# Create a signal with a type and data\nsignal = Signal.new(:user_action, %{id: 1, data: \"example\"})\n```","ref":"getting_started.html#1-signals"},{"type":"extras","title":"2. Primitives - Getting Started with AgentForge","doc":"Primitives are building blocks for creating handlers:\n\n```elixir\n# Transform primitive\ntransform = Primitives.transform(&String.upcase/1)\n\n# Branch primitive\nbranch = Primitives.branch(\n  fn signal, _ -> signal.data.age >= 18 end,\n  adult_handlers,\n  minor_handlers\n)\n\n# Loop primitive\nloop = Primitives.loop(fn item, state ->\n  {Signal.emit(:processed, item), state}\nend)\n\n# Wait primitive\nwait = Primitives.wait(\n  fn _, state -> state.ready end,\n  timeout: 5000\n)\n\n# Notify primitive\nnotify = Primitives.notify(\n  [:console],\n  format: &(\"Event: #{&1}\")\n)\n```","ref":"getting_started.html#2-primitives"},{"type":"extras","title":"3. Flows - Getting Started with AgentForge","doc":"Flows combine handlers into pipelines:\n\n```elixir\nworkflow = [\n  &validate/2,\n  &process/2,\n  &notify/2\n]\n\n{:ok, result, state} = Flow.process(workflow, signal, initial_state)\n```","ref":"getting_started.html#3-flows"},{"type":"extras","title":"Common Patterns - Getting Started with AgentForge","doc":"","ref":"getting_started.html#common-patterns"},{"type":"extras","title":"Error Handling - Getting Started with AgentForge","doc":"```elixir\n# In transforms\ntransform = Primitives.transform(fn data ->\n  # ... operation that might fail ...\nrescue\n  e -> raise \"Processing failed: #{Exception.message(e)}\"\nend)\n\n# In flows\ncase Flow.process(workflow, signal, state) do\n  {:ok, result, state} -> handle_success(result)\n  {:error, reason} -> handle_error(reason)\nend\n```","ref":"getting_started.html#error-handling"},{"type":"extras","title":"State Management - Getting Started with AgentForge","doc":"```elixir\n# Initialize state\nstate = %{counter: 0}\n\n# Update state in handler\ndef count(signal, state) do\n  new_state = Map.update(state, :counter, 1, & &1 + 1)\n  {signal, new_state}\nend\n```","ref":"getting_started.html#state-management"},{"type":"extras","title":"Configuration-based Workflows - Getting Started with AgentForge","doc":"AgentForge supports defining workflows in YAML:\n\n```yaml\nname: registration\nsteps:\n  - name: validate\n    type: transform\n    config:\n      validate:\n        - field: email\n          required: true\n        - field: password\n          min_length: 8\n  \n  - name: process\n    type: notify\n    config:\n      channels: [console]\n      message: \"New user: {email}\"\n```","ref":"getting_started.html#configuration-based-workflows"},{"type":"extras","title":"Next Steps - Getting Started with AgentForge","doc":"1. Read the [Core Concepts](core_concepts.md) guide for deeper understanding\n2. Check out the [examples/](../examples/) directory for more examples\n3. Learn about [extending primitives](extending_primitives.md)\n4. Review the [configuration guide](configuration.md)","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Help and Support - Getting Started with AgentForge","doc":"- Report issues on GitHub\n- Join the discussion in our community channels\n- Check the [documentation](https://hexdocs.pm/agent_forge) for detailed API references","ref":"getting_started.html#help-and-support"},{"type":"extras","title":"AgentForge Plugin System","doc":"# AgentForge Plugin System\n\nThis guide explains how to use and extend the AgentForge plugin system.","ref":"plugin_system.html"},{"type":"extras","title":"Overview - AgentForge Plugin System","doc":"The plugin system enables extending AgentForge with additional functionality while maintaining a lightweight core. Plugins can provide new tools, primitives, and notification channels.","ref":"plugin_system.html#overview"},{"type":"extras","title":"Using Plugins - AgentForge Plugin System","doc":"To use a plugin in your AgentForge application:\n\n1. Add the plugin to your dependencies in `mix.exs`:\n\n```elixir\ndefp deps do\n  [\n    {:agent_forge, \"~> 0.1.0\"},\n    {:agent_forge_http, \"~> 1.0.0\"}  # Example HTTP plugin\n  ]\nend\n```\n\n2. Load the plugin in your application:\n\n```elixir\n# In your application startup code\nAgentForge.PluginManager.load_plugin(AgentForge.Plugins.HTTP)\n```\n\n3. Use the tools provided by the plugin:\n\n```elixir\nsignal = AgentForge.Signal.new(:request, %{\"url\" => \"https://example.com\"})\nflow = [AgentForge.Tools.execute(\"http_get\")]\n{:ok, result, _} = AgentForge.Flow.process(flow, signal, %{})\n```","ref":"plugin_system.html#using-plugins"},{"type":"extras","title":"Creating Plugins - AgentForge Plugin System","doc":"To create your own plugin:\n\n1. Implement the `AgentForge.Plugin` behaviour:\n\n```elixir\ndefmodule MyApp.CustomPlugin do\n  @behaviour AgentForge.Plugin\n  \n  @impl true\n  def init(_opts) do\n    # Initialize your plugin\n    :ok\n  end\n  \n  @impl true\n  def register_tools(registry) do\n    # Register any tools your plugin provides\n    registry.register(\"my_tool\", &my_tool_function/1)\n    :ok\n  end\n  \n  @impl true\n  def metadata do\n    %{\n      name: \"My Custom Plugin\",\n      description: \"Provides custom functionality\",\n      version: \"1.0.0\",\n      author: \"Your Name\",\n      compatible_versions: \">= 0.1.0\"\n    }\n  end\n  \n  # Tool implementation\n  defp my_tool_function(params) do\n    # Process params and return a result\n    %{result: \"Processed #{inspect(params)}\"}\n  end\nend\n```\n\n2. Optionally register primitives or notification channels:\n\n```elixir\n# To register primitives\n@impl true\ndef register_primitives do\n  # Register your custom primitives\n  :ok\nend\n\n# To register notification channels\n@impl true\ndef register_channels do\n  AgentForge.Notification.Registry.register_channel(MyApp.Notification.Channels.Custom)\n  :ok\nend\n```","ref":"plugin_system.html#creating-plugins"},{"type":"extras","title":"Notification Channels - AgentForge Plugin System","doc":"Plugins can provide new notification channels by implementing the `AgentForge.Notification.Channel` behaviour:\n\n```elixir\ndefmodule MyApp.Notification.Channels.Custom do\n  @behaviour AgentForge.Notification.Channel\n  \n  @impl true\n  def name, do: :custom\n  \n  @impl true\n  def send(message, config) do\n    # Send notification through your custom channel\n    IO.puts(\"Custom notification: #{message}, config: #{inspect(config)}\")\n    :ok\n  end\nend\n```\n\nTo use a custom notification channel:\n\n```elixir\nnotify = AgentForge.Primitives.notify(\n  [:console, :custom],\n  config: %{custom: %{some_setting: \"value\"}}\n)\n\nflow = [notify]\nsignal = AgentForge.Signal.new(:event, \"Something happened\")\nAgentForge.Flow.process(flow, signal, %{})\n```","ref":"plugin_system.html#notification-channels"},{"type":"extras","title":"Best Practices - AgentForge Plugin System","doc":"1. **Keep Plugins Focused**: Each plugin should provide a specific, well-defined set of functionality.\n\n2. **Handle Dependencies Gracefully**: Check for required dependencies with `Code.ensure_loaded?/1` and provide meaningful error messages when they're missing.\n\n3. **Document Your Plugin**: Include clear documentation on how to use your plugin and its configuration options.\n\n4. **Version Compatibility**: Specify which versions of AgentForge your plugin is compatible with.\n\n5. **Testing**: Write comprehensive tests for your plugin to ensure it behaves correctly.","ref":"plugin_system.html#best-practices"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}